@@ -1,36 +1,21 @@
-/*
- * LEVEE, or Captain Video;  A vi clone
- *
- * Copyright (c) 1982-1997 David L Parsons
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms are permitted
- * provided that the above copyright notice and this paragraph are
- * duplicated in all such forms and that any documentation,
- * advertising materials, and other materials related to such
- * distribution and use acknowledge that the software was developed
- * by David L Parsons (orc@pell.chi.il.us).  My name may not be used
- * to endorse or promote products derived from this software without
- * specific prior written permission.  THIS SOFTWARE IS PROVIDED
- * AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
- * WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND
- * FITNESS FOR A PARTICULAR PURPOSE.
- */
-#include <ctype.h>
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-
 #include "levee.h"
 #include "extern.h"
+#include <string.h>
+#include <stdlib.h>
+
+VOID PROC undefine();
+VOID PROC fixupline();
+VOID PROC doinput();
 
 /*
  * do a newline and set flags.
  */
 #define exprintln()	(zotscreen=YES),println()
 
-/* Pretty fancy, eh? */
-void plural(int num, char *string)
+VOID PROC
+plural(num,string)
+int num;
+char *string;
 {
     printi(num);
     prints(string);
@@ -39,16 +24,19 @@ void plural(int num, char *string)
 } /* plural */
 
 
-void clrmsg()
+VOID PROC
+clrmsg()
 {
-    levee_mvcur(-1, 0);
+    mvcur(-1,0);
     strput(CE);
 } /* clrmsg */
 
 
-void errmsg(char *msg)
+VOID PROC
+errmsg(msg)
+char *msg;
 {
-    levee_mvcur(-1, 0);
+    mvcur(-1,0);
     prints(msg);
     strput(CE);
 } /* errmsg */
@@ -57,7 +45,8 @@ void errmsg(char *msg)
 /* get a space-delimited token */
 char *execstr;			/* if someone calls getarg in the	*/
 				/* wrong place, death will come...	*/
-char *getarg()
+char *PROC
+getarg()
 {
     char *rv;
     rv = execstr;
@@ -72,43 +61,57 @@ char *getarg()
 } /* getarg */
 
 
-void version()
+VOID PROC
+version()
 /* version: print which version of levee we are... */
 {
-    errmsg("new-levee ");prints(ED_NOTICE);printch(ED_REVISION);
+    errmsg("levee (c)");prints(codeversion);
 } /* version */
 
 
-void args()
+VOID PROC
+args()
 /* args: print the argument list */
 {
-    int i;
-    levee_mvcur(-1,0);
+    register int i;
+    mvcur(-1,0);
     for (i=0; i < argc; i++) {
 	if (curpos.x+strlen(argv[i]) >= COLS)
 	    exprintln();
 	else if (i > 0)
 	    printch(' ');
 	if (pc == i) {			/* highlight the current filename.. */
+#if OS_ATARI|OS_FLEXOS
+	    strput("\033p");
+#else
 	    printch('[');
+#endif
 	    prints(argv[i]);
+#if OS_ATARI|OS_FLEXOS
+	    strput("\033q");
+#else
 	    printch(']');
+#endif
 	}
 	else
 	    prints(argv[i]);
     }
 } /* args */
     
-void setcmd()
+VOID PROC
+setcmd()
 {
-    boolean no = YES, b;
+    bool no = NO,b;
+#if 0
+    int len,i;
+#endif
     char *arg, *num;
     struct variable *vp;
     
-    if ((arg = getarg())) {
+    if ( (arg = getarg()) ) {
 	do {
 	    if (*arg != 0) {
-		if ((num = strchr(arg, '='))) {
+		if ( (num = strchr(arg, '=')) ) {
 		    b = NO;
 		    *num++ = 0;
 		}
@@ -117,16 +120,18 @@ void setcmd()
 		    if (arg[0]=='n' && arg[1]=='o') {
 			arg += 2;
 			no = NO;
-		    } else {
-			no = YES;
 		    }
+		    else
+			no = YES;
 		}
 		for(vp=vars;vp->u && strcmp(arg,vp->v_name)
-				  && strcmp(arg,vp->v_abbr); vp++);
+				  && strcmp(arg,vp->v_abbr); vp++)
+		    ;
 		if (!vp->u || vp->v_flags & V_CONST) {
-		    errmsg("can't set ");
+		    errmsg("Can't set ");
 		    prints(arg);
-		} else {
+		}
+		else {
 		    int j;
                     
                     if (b)
@@ -150,7 +155,7 @@ void setcmd()
 		    diddled |= vp->v_flags & V_DISPLAY;
 		}
 	    }
-	} while ((arg = getarg()));
+	} while ( (arg = getarg()) );
     }
     else {
 	version(); exprintln();
@@ -166,14 +171,14 @@ void setcmd()
 			prints("no ");
 		    prints(vp->v_name);
 		    if (vp->u->strp) {
-			levee_mvcur(-1,10);
+			mvcur(-1,10);
 			prints("= ");
 			prints(vp->u->strp);
 		    }
 		    break;
 		default:
 		    prints(vp->v_name);
-		    levee_mvcur(-1,10);
+		    mvcur(-1,10);
 		    prints("= ");
 		    printi(vp->u->valu);
 		    break;
@@ -185,13 +190,15 @@ void setcmd()
 
 
 /* print a macro */
-void printone(int i)
+VOID PROC
+printone(i)
+int i;
 {
     if (i >= 0) {
 	exprintln();
 	printch(mbuffer[i].token);
-	levee_mvcur(-1,3);
-	if (movemap[(int)mbuffer[i].token] == INSMACRO)
+	mvcur(-1,3);
+	if (movemap[(unsigned int)(mbuffer[i].token)] == INSMACRO)
 	    prints("!= ");
 	else
 	    prints(" = ");
@@ -201,7 +208,8 @@ void printone(int i)
 
 
 /* print all the macros */
-void printall()
+VOID PROC
+printall()
 {
     int i;
     for (i = 0; i < MAXMACROS; i++)
@@ -211,18 +219,20 @@ void printall()
 
 
 /* :map ch text */
-int map(boolean insert)
+VOID PROC
+map(insert)
+bool insert;
 {
     char *macro, c;
     int i;
     		/* get the macro */
     if ((macro=getarg()) == NULL) {
 	printall();
-	return 1;
+	return;
     }
     if (strlen(macro) > 1) {
 	errmsg("macros must be one character");
-	return 0;
+	return;
     }
     c = macro[0];
     if (*execstr == 0)
@@ -238,22 +248,23 @@ int map(boolean insert)
 	    undefine(lookup(c));
 	    mbuffer[i].token = c;
 	    mbuffer[i].m_text = strdup(execstr);
-	    mbuffer[i].oldmap = movemap[(int)c];
+	    mbuffer[i].oldmap = movemap[(unsigned int)c];
 	    if (insert)
-		movemap[(int)c] = INSMACRO;
+		movemap[(unsigned int)c] = INSMACRO;
 	    else
-		movemap[(int)c] = SOFTMACRO;
+		movemap[(unsigned int)c] = SOFTMACRO;
 	}
     }
-    return 0; /* FIXME: not reached (?) */
 } /* map */
 
 
-void undefine(int i)
+VOID PROC
+undefine(i)
+int i;
 {
     char *p;
     if (i >= 0) {
-	movemap[(int)mbuffer[i].token] = mbuffer[i].oldmap;
+	movemap[(unsigned int)(mbuffer[i].token)] = mbuffer[i].oldmap;
 	mbuffer[i].token = 0;
 	p = mbuffer[i].m_text;
 	free(p);
@@ -262,12 +273,13 @@ void undefine(int i)
 } /* undefine */
 
 
-boolean unmap()
+int PROC
+unmap()
 {
     int i;
     char *arg;
     
-    if ((arg=getarg())) {
+    if ( (arg=getarg()) ) {
 	if (strlen(arg) == 1) {
 	    undefine(lookup(*arg));
 	    return YES;
@@ -284,7 +296,9 @@ boolean unmap()
 
 
 /* return argument # of a filename */
-int findarg(char *name)
+int PROC
+findarg(name)
+register char *name;
 {
     int j;
     for (j = 0; j < argc; j++)
@@ -295,7 +309,9 @@ int findarg(char *name)
 
 
 /* add a filename to the arglist */
-int addarg(char *name)
+int PROC
+addarg(name)
+register char *name;
 {
     int where;
     if ((where = findarg(name)) < 0)
@@ -305,41 +321,54 @@ int addarg(char *name)
 
 
 /* get a file name argument (substitute alt file for #) */
-char *getname()
+char * PROC
+getname()
 {
     extern int wilderr;
-    char *name;
-    if ((name = getarg())) {
-	if (strcmp(name,"#") == 0) {
-	    if (*altnm) {
+#if OS_ATARI
+    extern int mapslash;
+    register char *p;
+#endif
+    register char *name;
+
+    if ( (name = getarg()) ) {
+	if ( 0 == strcmp(name,"#") ) {
+	    if (*altnm)
 		name = altnm;
-	    } else {
+	    else {
 		errmsg("no alt name");
 		wilderr++;
 		return NULL;
 	    }
 	}
+#if OS_ATARI
+	if (mapslash)
+	    for (p=name; *p; p++)
+		if (*p == '/')
+		    *p = '\\';
+#endif
     }
     return name;
 } /* getname */
 
 
 /* CAUTION: these make exec not quite recursive */
-int  high, low;		/* low && high end of command range */
-boolean affirm;		/* cmd! (use of ! to override */
-
+int  high,low;		/* low && high end of command range */
+bool affirm;		/* cmd! */
 /* s/[src]/dst[/options] */
 /* s& */
-void cutandpaste()
+VOID PROC
+cutandpaste()
 {
-    boolean askme  = NO,
+    bool askme  = NO,
 	 printme= NO,
 	 glob   = NO;
     int newcurr = -1;
     int oldcurr = curr;
     int  num;
     char delim;
-    char *ip, *dp;
+    register char *ip;
+    register char *dp;
     
     zerostack(&undo);
     ip = execstr;
@@ -394,93 +423,131 @@ splat:	errmsg("bad substitute");
 } /* cutandpaste */
 
 
-void inputf(char *fname, boolean newbuf)
+/* quietly read in a file (and mark it in the undo stack)
+ */
+int PROC
+insertfile(FILE *f, int insert, int at, int *fsize)
 {
-    int onright = 0,	/* text moved right for :read */
-	fsize;		/* bytes read in */
-    FILE *f;
+    int high,
+	onright,
+	rc=0;
 
+    onright = (bufmax-at);
+    high = SIZE-onright;
+
+    if ( insert && (onright > 0) )
+	 moveright(&core[at], &core[high], onright);
+
+    rc = addfile(f, at, high, fsize);
+
+    if ( (rc == 0) && (*fsize < 0) ) {
+	rc = -1;
+	*fsize=0;
+    }
+    if ( insert ) {
+	if ( *fsize ) 
+	    insert_to_undo(&undo, at, *fsize);
+	modified = YES;
+	if (onright > 0)
+	    moveleft(&core[high], &core[at+(*fsize)], onright);
+    }
+    diddled = YES;
+    return rc;
+} /* insertfile */
+
+
+
+VOID PROC
+inputf(fname, newbuf)
+register char *fname;
+bool newbuf;
+{
+    FILE *f;
+    int fsize,		/* bytes read in */
+	rc;
 
     if (newbuf)
 	readonly = NO;
-
+	
     zerostack(&undo);
-    if (newbuf) {
+
+    if ( newbuf ) {
 	modified = NO;
 	low = 0;
-	high = SIZE;
-    } else {		/* append stuff to the buffer */
-	fixupline(bseekeol(curr));
-	onright = bufmax-low;
-	high = (SIZE-onright);
-	if (onright > 0)
-	    moveright(&core[low], &core[high], onright);
     }
-
+    else {
+	fixupline(bseekeol(curr));
+    }
+    
     printch('"');
     prints(fname);
     prints("\" ");
-
     if ((f=fopen(fname, "r")) == NULL) {
-	prints("[no such file]");
+	prints("[No such file]");
 	fsize = 0;
 	if (newbuf)
 	    newfile = YES;
-    } else {
-	if (addfile(f, low, high, &fsize))
-	    plural(fsize," byte");
-	else if (fsize < 0) {
-	    prints("[read error]");
-	    fsize = 0;
-	} else {
-	    prints("[overflow]");
-	    readonly = YES;
-	}
-	fclose(f);
-	if (newbuf)
-	    newfile = NO;
     }
+    else {
+	rc = insertfile(f, !newbuf, low, &fsize);
+	fclose(f);
 
+	if ( rc > 0 )
+	    plural(fsize, " byte");
+	else if ( rc < 0 )
+	    prints("[read error]");
+	else {
+	    prints("[overflow]");
+	    readonly=1;
+	}
+	if (newbuf) newfile = NO;
+    }
     if (newbuf) {
 	fillchar(contexts, sizeof(contexts), -1);
 	bufmax = fsize;
-    } else {
-	insert_to_undo(&undo, low, fsize);
-	modified = YES;
-	if (onright > 0)
-	    moveleft(&core[high], &core[low+fsize], onright);
     }
-
     if (*startcmd) {
 	count = 1;
-
-	if (findparse(startcmd, &curr, low)[0] || curr < 0)
+	if (*findparse(startcmd,&curr,low) != 0 || curr < 0)
 	    curr = low;
-
 	*startcmd = 0;
-    } else {
-	curr = low;
     }
-    diddled = YES;
+    else
+	curr = low;
 } /* inputf */
 
 
 /* Change a file's name (for autocopy). */
-void backup(char *name)
+VOID PROC
+backup(name)
+char *name;
 {
     char back[80];
+#if !OS_UNIX
+    char *p;
+#endif
 
     strcpy(back, name);
+#if OS_UNIX
     strcat(back, "~");
+#else
+    p = strrchr(basename(back), '.');
+    if (p)
+	strcpy(1+p, ",bkp");
+    else
+	strcat(back, ".bkp");
+#endif
     
     unlink(back);
     rename(name, back);
 } /* backup */
 
 
-boolean outputf(char *fname)
+bool PROC
+outputf(fname)
+char *fname;
 {
-    boolean whole;
+    bool whole;
     FILE *f;
     int status;
     zerostack(&undo);		/* undo doesn't survive past write */
@@ -494,7 +561,7 @@ boolean outputf(char *fname)
     whole = (low == 0 && high >= bufmax-1);
     if (whole && autocopy)
 	backup(fname);
-    if ((f = fopen(fname, "w"))) {
+    if ( (f=fopen(fname, "w")) ) {
 	status = putfile(f, low, high);
 	fclose(f);
 	if (status) {
@@ -514,18 +581,22 @@ boolean outputf(char *fname)
 } /* outputf */
 
 
+int PROC
+oktoedit(writeold)
 /* check and see if it is ok to edit a new file */
-boolean oktoedit(boolean writeold) /* automatically write out changes? */
+int writeold;	/* automatically write out changes? */
 {
     if (modified && !affirm) {
 	if (readonly) {
 	    errmsg(fisro);
 	    return NO;
-	} else if (writeold && *filenm) {
+	}
+	else if (writeold && *filenm) {
 	    if (!outputf(filenm))
 		return NO;
 	    printch(',');
-	} else {
+	}
+	else {
 	    errmsg(fismod);
 	    return NO;
 	}
@@ -535,7 +606,8 @@ boolean oktoedit(boolean writeold) /* automatically write out changes? */
 
 
 /* write out all || part of a file */
-boolean writefile()
+bool PROC
+writefile()
 {
     char *name;
     
@@ -555,7 +627,8 @@ boolean writefile()
 }
 
 
-void editfile()
+VOID PROC
+editfile()
 {
     char *name = NULL;	/* file to edit */
     char **myargv;
@@ -570,7 +643,7 @@ void editfile()
 	do {
 	    if (!expandargs(name, &myargc, &myargv))
 		return;
-	} while ((name = getarg()));
+	} while ( (name=getarg()) );
     if (myargc == 0) {
 	if (*filenm)
 	    name = filenm;
@@ -588,7 +661,9 @@ void editfile()
 }
 
 
-void doinput(char *name)
+VOID PROC
+doinput(name)
+char *name;
 {
     inputf(name, YES);
     strcpy(altnm, filenm);
@@ -596,7 +671,9 @@ void doinput(char *name)
 }
 
 
-void toedit(int count)
+VOID PROC
+toedit(count)
+int count;
 {
     if (count > 1) {
 	printi(count);
@@ -605,26 +682,30 @@ void toedit(int count)
 }
 
 
-void readfile()
+VOID PROC
+readfile()
 {
     char *name;
     
-    if ((name = getarg()))
+    if ( (name=getarg()) )
 	inputf(name,NO);
     else
 	errmsg("no file to read");
 }
 
 
-void nextfile(boolean prev)
+VOID PROC
+nextfile(prev)
+bool prev;
 {
     char *name = NULL;
-    int newpc = pc, myargc = 0;
+    int newpc=pc,
+	myargc=0;
     char **myargv;
-    boolean newlist = NO;
+    bool newlist = NO;
     
     if (prev == 0)
-	while ((name = getarg()))
+	while ( (name=getarg()) )
 	    if (!expandargs(name, &myargc, &myargv))
 		return;
     
@@ -669,7 +750,9 @@ void nextfile(boolean prev)
 /*
  * set up low, high; set dot to low
  */
-void fixupline(int dft)
+VOID PROC
+fixupline(dft)
+int dft;
 {
     if (low < 0)
 	low = dft;
@@ -692,7 +775,8 @@ void fixupline(int dft)
 }
 
 
-void whatline()
+VOID PROC
+whatline()
 {
     printi(to_line((low < 0) ? (bufmax-1) : low));
     if (high >= 0) {
@@ -702,7 +786,8 @@ void whatline()
 }
 
 
-void print()
+VOID PROC
+print()
 {
     do {
 	exprintln();
@@ -716,7 +801,11 @@ void print()
 /* execute lines from a :sourced || .lvrc file */
 
 
-boolean do_file(char *fname, exec_type *mode, boolean *noquit)
+bool PROC
+do_file(fname,mode,noquit)
+char *fname;
+exec_type *mode;
+bool *noquit;
 {
     char line[120];
     FILE *fp, *fopen();
@@ -736,7 +825,9 @@ boolean do_file(char *fname, exec_type *mode, boolean *noquit)
 }
 
 
-void doins(boolean flag)
+VOID PROC
+doins(flag)
+bool flag;
 {
     int i;
     curr = low;
@@ -749,7 +840,9 @@ void doins(boolean flag)
 
 
 /* figure out a address range for a command */
-char *findbounds(char *ip)
+char * PROC
+findbounds(ip)
+char *ip;
 {
     ip = findparse(ip, &low, curr);	/* get the low address */
     if (low >= 0) {
@@ -771,9 +864,11 @@ char *findbounds(char *ip)
 
 
 /* parse the command line for lineranges && a command */
-exec_mode_command parse(char *inp)
+int PROC
+parse(inp)
+char *inp;
 {
-    int j, k;
+    int j,k;
     char cmd[80];
     low = high = ERR;
     affirm = 0;
@@ -817,10 +912,14 @@ exec_mode_command parse(char *inp)
 
 
 /* inner loop of execmode */
-void exec(char *cmd, exec_type *mode, boolean *noquit)
+VOID PROC
+exec(cmd, mode, noquit)
+char *cmd;
+exec_type *mode;
+bool *noquit;
 {
-    exec_mode_command what;
-    boolean ok;
+    int  what;
+    bool ok;
     
     what = parse(cmd);
     ok = YES;
@@ -830,7 +929,7 @@ void exec(char *cmd, exec_type *mode, boolean *noquit)
     }
     switch (what) {
 	case EX_QUIT:				/* :quit */
-	    if (affirm || !modified)
+	    if (affirm || what == lastexec || !modified)
 		*noquit = NO;
 	    else
 		errmsg(fismod);
@@ -871,9 +970,9 @@ void exec(char *cmd, exec_type *mode, boolean *noquit)
 		if (readonly) {
 		    prints(fisro);
 		    break;
-		} else if (!writefile()) {
-		    break;
 		}
+		else if (!writefile())
+		    break;
 	    }
 
 	    if (!affirm && (argc-pc > 1)) {	/* any more files to edit? */
@@ -891,7 +990,7 @@ void exec(char *cmd, exec_type *mode, boolean *noquit)
 	    ok = unmap();
 	    break;
 	case EX_FILE:				/* :file */
-	    if ((cmd = getarg())) {		/* :file name */
+	    if ( (cmd=getarg()) ) {		/* :file name */
 		strcpy(altnm, filenm);
 		strcpy(filenm, cmd);
 		pc = addarg(filenm);
@@ -912,7 +1011,7 @@ void exec(char *cmd, exec_type *mode, boolean *noquit)
 	    break;
 	case EX_DELETE:
 	case EX_YANK:				/* :delete, :yank */
-	    yank.line = YES;
+	    yank.lines = YES;
 	    fixupline(lstart);
 	    zerostack(&undo);
 	    if (what == EX_DELETE)
@@ -952,7 +1051,7 @@ void exec(char *cmd, exec_type *mode, boolean *noquit)
 		ok = NO;		/* kludge, kludge, kludge!!!!!!!!!! */
 	    else {
 		zerostack(&undo);
-		yank.line = YES;
+		yank.lines = YES;
 		fixupline(lstart);
 		if (deletion(low,high))
 		    doins(YES);
@@ -978,12 +1077,20 @@ void exec(char *cmd, exec_type *mode, boolean *noquit)
 	    zotscreen = YES;
 	    exprintln();
 	    if (*execstr) {
-#if ZTERM
+#if TTY_ZTERM
 		zclose();
 #endif
+#if OS_FLEXOS|OS_UNIX
 		fixcon();
+#else
+		allowintr();
+#endif
 		system(execstr);
+#if OS_FLEXOS|OS_UNIX
 		initcon();
+#else
+		nointr();
+#endif
 	    }
 	    else
 		prints("incomplete shell escape.");
@@ -999,6 +1106,7 @@ void exec(char *cmd, exec_type *mode, boolean *noquit)
 	    prints(":not an editor command.");
 	    break;
     }
+    lastexec = what;
     if (!ok) {
 	errmsg(excmds[what]);
 	prints(" error");
